package main

import (
	"errors"
	"fmt"
	"git.user-penguin.space/vladimir/all-tg-bot/commands"
	"git.user-penguin.space/vladimir/all-tg-bot/db"
	tgbot "github.com/go-telegram-bot-api/telegram-bot-api"
	"github.com/umputun/go-flags"
	"io/ioutil"
	"log"
	"os"
	"strings"
)

const tag = "@all"

type Opts struct {
	Token string `short:"t" long:"token" description:"Telegram api token"`
	Name  string `short:"n" long:"name" description:"Telegram bot name" default:"@butler_tg_bot"`
}

var opts Opts

func readToken() (string, error) {
	b, err := ioutil.ReadFile("token.txt")
	if err != nil {
		return "", err
	}
	str := string(b)
	return str, nil
}

func main() {
	run()
}

func run() {
	p := flags.NewParser(&opts, flags.PrintErrors|flags.PassDoubleDash|flags.HelpFlag)
	p.SubcommandsOptional = true
	if _, err := p.Parse(); err != nil {
		if err.(*flags.Error).Type != flags.ErrHelp {
			log.Println(errors.New("[ERROR] cli error: " + err.Error()))
		}
		os.Exit(2)
	}

	if opts.Token == "" {
		token, err := readToken()
		if err != nil {
			panic(err)
		}
		opts.Token = strings.ReplaceAll(token, "\n", "")
	}
	fmt.Println(opts.Token)

	bot, err := tgbot.NewBotAPI(opts.Token)
	if err != nil {
		panic(err)
	}

	u := tgbot.NewUpdate(0)
	u.Timeout = 60

	updates, err := bot.GetUpdatesChan(u)
	if err != nil {
		panic(err)
	}

	log.Println("Run", opts.Name)

	for update := range updates {

		if update.Message == nil {
			continue
		}

		text := update.Message.Text
		chatID := update.Message.Chat.ID
		username := update.Message.From.UserName

		_ = db.AddUserInChat(chatID, username)

		if strings.Contains(strings.ToLower(text), tag) {
			users, _ := db.GetUsers(chatID)
			s := ""
			i := 0
			for _, user := range users {
				if username == user {
					continue
				}
				if i > 0 {
					s += " "
				}
				s += "@" + user
				i++
			}
			_, _ = bot.Send(tgbot.NewMessage(chatID, s))
		}

		command := commands.Command(strings.Replace(text, opts.Name, "", 1))
		switch command {
		case commands.Start:
			_, _ = bot.Send(tgbot.NewMessage(chatID, fmt.Sprintf("Здравствуйте, я Волтер (v1.2.5), ваш дворецкий (%d).", chatID)))
			_ = db.AddChat(chatID)
		case commands.Help:
			_, _ = bot.Send(tgbot.NewMessage(chatID, "Вот что я умею:\n\n1) Позвать всех"))
		case commands.Ping:
			_, _ = bot.Send(tgbot.NewMessage(chatID, "pong"))
		}
	}
}
